==> koishi-plugin-lovemilk-telemetry-0.0.1.tgz: package/lib/index.js <==
var __defProp = Object.defineProperty;
var __getOwnPropDesc = Object.getOwnPropertyDescriptor;
var __getOwnPropNames = Object.getOwnPropertyNames;
var __hasOwnProp = Object.prototype.hasOwnProperty;
var __name = (target, value) => __defProp(target, "name", { value, configurable: true });
var __export = (target, all) => {
  for (var name2 in all)
    __defProp(target, name2, { get: all[name2], enumerable: true });
};
var __copyProps = (to, from, except, desc) => {
  if (from && typeof from === "object" || typeof from === "function") {
    for (let key of __getOwnPropNames(from))
      if (!__hasOwnProp.call(to, key) && key !== except)
        __defProp(to, key, { get: () => from[key], enumerable: !(desc = __getOwnPropDesc(from, key)) || desc.enumerable });
  }
  return to;
};
var __toCommonJS = (mod) => __copyProps(__defProp({}, "__esModule", { value: true }), mod);

// src/index.ts
var src_exports = {};
__export(src_exports, {
  Client: () => Client,
  ClientType: () => ClientType,
  Config: () => Config,
  LovemilkTelemetry: () => LovemilkTelemetry,
  apply: () => apply,
  getErrorDetail: () => getErrorDetail,
  inject: () => inject,
  makeEvent: () => makeEvent,
  name: () => name
});
module.exports = __toCommonJS(src_exports);
var import_koishi3 = require("koishi");

// src/telemetry/client.ts
var import_koishi2 = require("koishi");

// src/telemetry/types.ts
var ClientType = /* @__PURE__ */ ((ClientType2) => {
  ClientType2["koishi"] = "koishi";
  ClientType2["nonebot"] = "nonebot";
  return ClientType2;
})(ClientType || {});

// src/shared.ts
var import_koishi = require("koishi");
var import_node_machine_id = require("node-machine-id");
var logger = new import_koishi.Logger("lovemilk-telemetry");
var machineId = (0, import_node_machine_id.machineIdSync)(true);

// src/telemetry/client.ts
var defaultHeartbeatInterval = import_koishi2.Time.minute;
function makeEvent(name2, clientType, mid, data) {
  const now = /* @__PURE__ */ new Date();
  const UTCNow = new Date(
    now.getUTCFullYear(),
    now.getUTCMonth(),
    now.getUTCDate(),
    now.getUTCHours(),
    now.getUTCMinutes(),
    now.getUTCSeconds(),
    now.getUTCMilliseconds()
  );
  return {
    name: name2,
    localTime: now.getTime(),
    localUTCTime: UTCNow.getTime(),
    client: {
      type: clientType,
      machineID: mid,
      version: import_koishi2.version
      // imported from 'koishi'
    },
    data
  };
}
__name(makeEvent, "makeEvent");
function getErrorDetail(resp) {
  return resp.data ? resp.data["detail"] ?? resp.data["error"] ?? "" : "";
}
__name(getErrorDetail, "getErrorDetail");
var Client = class _Client {
  static {
    __name(this, "Client");
  }
  type = "koishi" /* koishi */;
  mid = machineId;
  pluginCtx;
  pluginConfig;
  ctx;
  static config;
  endpoint;
  connection = null;
  events = [];
  connected = false;
  disabled = false;
  static timeout = 3e4;
  heartbeatInterval = defaultHeartbeatInterval;
  heartbeatTimeoutID;
  isReconnecting = false;
  lastReconnect = 0;
  reconnectTried = 0;
  AutoReconnect = async (event = null, force = false) => {
    if (this.isReconnecting || this.disabled || !this.connection) {
      return;
    }
    this.isReconnecting = true;
    const core = /* @__PURE__ */ __name(async () => {
      logger.debug("telemetry connection lost, reconnecting...");
      try {
        await this.connect();
        this.isReconnecting = false;
      } catch (e) {
        this.lastReconnect = Date.now();
        this.reconnectTried += 1;
        this.isReconnecting = false;
        logger.debug(`telemetry reconnect failed
${e.stack}`);
        setTimeout(async () => {
          await this.AutoReconnect();
        }, 0);
      }
    }, "core");
    if (force) {
      this.ctx.setTimeout(async () => {
        await core();
      }, 0);
      return;
    }
    if (this.reconnectTried > _Client.config.telemetryMaxTry) {
      logger.warn(`telemetry reconnect failed, tried ${this.reconnectTried} times > ${_Client.config.telemetryMaxTry}, will retry in ${_Client.config.telemetryCoolDown} ms`);
      if (this.connection) {
        this.connection.removeEventListener("close", this.AutoReconnect);
      }
      this.connected = false;
      await (0, import_koishi2.sleep)(_Client.config.telemetryCoolDown);
      this.isReconnecting = false;
      this.reconnectTried = 0;
      this.lastReconnect = 0;
      await core();
      return;
    }
    const deltatime = Date.now() - this.lastReconnect;
    if (this.reconnectTried > _Client.config.telemetryUseTimeoutMoreThan) {
      if (deltatime < _Client.config.telemetryReconnectInterval) {
        logger.warn(`telemetry reconnect failed too many in ${this.reconnectTried} times, will retry in ${_Client.config.telemetryReconnectInterval - deltatime} ms`);
        this.connected = false;
        await (0, import_koishi2.sleep)(_Client.config.telemetryReconnectInterval - deltatime);
        await core();
        return;
      }
    } else {
      if (deltatime < _Client.timeout) {
        this.connected = false;
        await (0, import_koishi2.sleep)(_Client.timeout - deltatime);
      }
      await core();
      return;
    }
  };
  onSend = (_, __) => {
  };
  constructor(ctx, pluginCtx, pluginConfig, endpoint) {
    this.ctx = ctx;
    this.pluginCtx = pluginCtx;
    this.pluginConfig = pluginConfig;
    this.endpoint = new URL(endpoint);
  }
  static _setConfig(config) {
    _Client.config = config;
    _Client.timeout = _Client.config.axiosConfig ? _Client.config.axiosTimeout : 3e4;
  }
  isConnected() {
    return this.connected;
  }
  appendEventHistory(eventOrevents) {
    const events = Array.isArray(eventOrevents) ? eventOrevents : [eventOrevents];
    this.events.unshift(...events);
  }
  enable() {
    logger.success("telemetry is enabled");
    this.disabled = false;
  }
  disable() {
    logger.success("telemetry is disabled");
    this.disabled = true;
  }
  heartbeat() {
    if (!this.connected || this.disabled) {
      return;
    }
    this.heartbeatTimeoutID = setTimeout(async () => {
      if (this.connected) {
        try {
          await this.sendEvent("ping");
        } catch (e) {
          logger.error(`telemetry ping error:
${e.stack}`);
          if (!this.isReconnecting) {
            this.ctx.setTimeout(() => {
              this.AutoReconnect(null, true);
            }, 0);
          }
        }
      }
      if (this.connection) {
        this.heartbeat();
      }
    }, this.heartbeatInterval);
  }
  async connect() {
    if (this.disabled) {
      return;
    }
    if (this.connected) {
      this.connection.removeEventListener("close", this.AutoReconnect);
    }
    this.connection = await this.ctx.http.ws(this.endpoint, {
      timeout: _Client.timeout
    });
    return new Promise((resolve, reject) => {
      const openListener = /* @__PURE__ */ __name(async (event) => {
        clearTimeout(timeoutID);
        this.reconnectTried = 0;
        this.lastReconnect = 0;
        this.connected = true;
        this.connection.removeEventListener("open", openListener);
        this.connection.addEventListener("close", this.AutoReconnect);
        let resp;
        try {
          resp = await this.sendEvent("online", { isReconnect: this.isReconnecting, reconnectTimes: this.reconnectTried + (this.isReconnecting ? 1 : 0), config: this.pluginConfig });
        } catch (e) {
          reject(e);
          return;
        }
        this.heartbeatInterval = resp.data?.heartbeat_interval * 1e3 / 2;
        logger.debug(`heartbeat interval set: ${this.heartbeatInterval} ms`);
        logger.debug("telemetry handshake succeed");
        this.heartbeat();
        resolve(resp);
      }, "openListener");
      this.connection.addEventListener("open", openListener);
      const errorListener = /* @__PURE__ */ __name((event) => {
        clearTimeout(timeoutID);
        this.connected = false;
        this.connection.removeEventListener("error", errorListener);
        this.ctx.setTimeout(async () => {
          await this.AutoReconnect();
        }, 0);
        reject(event.error);
      }, "errorListener");
      this.connection.addEventListener("error", errorListener);
      const timeoutID = setTimeout(() => {
        this.connection.removeEventListener("open", openListener);
        reject(new Error(`waiting server response timeout of ${_Client.timeout} ms`));
      }, _Client.timeout);
    });
  }
  async disconnect() {
    if (this.disabled) {
      return;
    }
    if (this.connection) {
      this.connection.removeEventListener("close", this.AutoReconnect);
      this.connection.removeEventListener("error", this.AutoReconnect);
      clearTimeout(this.heartbeatTimeoutID);
      let wavehandError = void 0;
      try {
        await this.sendEvent("offline", { config: this.pluginConfig }, true);
      } catch (e) {
        wavehandError = e;
        logger.warn(`telemetry failed to wavehand:
${e.stack}`);
      }
      this.connected = false;
      await this.connection.close();
      return new Promise((resolve, reject) => {
        wavehandError === void 0 ? resolve() : reject(wavehandError);
      });
    }
  }
  async sendEvent(name2, data, ignoreResp = false) {
    if (this.disabled) {
      return;
    }
    const event = makeEvent(name2, this.type, this.mid, data);
    this.events.push(event);
    this.onSend(event, this.events);
    if (!this.connected) {
      logger.warn(`telemetry client is not connected while sending ${name2} event`);
      return;
    }
    logger.debug("telemetry ↑", event);
    await this.connection.send(JSON.stringify(event));
    if (ignoreResp) {
      return;
    }
    return new Promise((resolve, reject) => {
      const messageListener = /* @__PURE__ */ __name((event2) => {
        clearTimeout(timeoutID);
        if (this.connection) {
          this.connection.removeEventListener("message", messageListener);
        }
        try {
          const received = JSON.parse(event2.data);
          logger.debug("telemetry ↓", received);
          resolve(received);
        } catch (e) {
          reject(e);
        }
      }, "messageListener");
      this.connection.addEventListener("message", messageListener);
      const errorListener = /* @__PURE__ */ __name((event2) => {
        clearTimeout(timeoutID);
        if (this.connection) {
          this.connection.removeEventListener("error", errorListener);
        }
        reject(event2.error);
      }, "errorListener");
      this.connection.addEventListener("error", errorListener);
      const timeoutID = setTimeout(() => {
        if (this.connection) {
          this.connection.removeEventListener("message", messageListener);
          this.connection.removeEventListener("error", errorListener);
        }
        reject(new Error(`waiting server response timeout of ${_Client.timeout} ms`));
      }, _Client.timeout);
    });
  }
};

// src/index.ts
var name = "lovemilk-telemetry";
var inject = ["http"];
var isDev = process.env.NODE_ENV === "development";
var Config = import_koishi3.Schema.intersect([
  import_koishi3.Schema.object({
    telemetryReconnectInterval: import_koishi3.Schema.number().min(1).max(import_koishi3.Time.day).default(import_koishi3.Time.minute * 15).description("遥测重连间隔 (毫秒)").step(1).hidden(!isDev),
    telemetryUseTimeoutMoreThan: import_koishi3.Schema.number().min(3).default(10).description("当遥测连续重连失败 (不包括首次连接) 大于该值后, 使用 `telemetryReconnectInterval` 毫秒 遥测重连间隔, 否则, 使用超时时间作为重连间隔").step(1).hidden(!isDev),
    telemetryMaxTry: import_koishi3.Schema.number().min(10).default(20).description("当遥测连续重连失败 (不包括首次连接) 大于该值后, 暂停 `telemetryCoolDown` 毫秒后重制重连状态").step(1).hidden(!isDev),
    telemetryCoolDown: import_koishi3.Schema.number().min(import_koishi3.Time.minute * 5).default(import_koishi3.Time.hour * 12).description("当遥测连续重连失败 (不包括首次连接) 大于 `telemetryMaxTry` 后, 暂停 该值(毫秒) 后重制重连状态").step(1).hidden(!isDev)
  }).description("遥测配置"),
  import_koishi3.Schema.object({
    axiosConfig: import_koishi3.Schema.boolean().default(false).description("启用自定义请求配置 <br>> 不启用将使用默认请求配置")
  }).description("请求配置"),
  import_koishi3.Schema.union([
    import_koishi3.Schema.object({
      axiosConfig: import_koishi3.Schema.const(true).required(),
      axiosTimeout: import_koishi3.Schema.number().default(3e5).description("请求超时 毫秒")
    }),
    import_koishi3.Schema.object({})
  ])
]);
function apply(ctx, config) {
  logger.debug(`machine ID: ${machineId}`);
  Client._setConfig(config);
  ctx.plugin(LovemilkTelemetry, config);
}
__name(apply, "apply");
var LovemilkTelemetry = class extends import_koishi3.Service {
  static {
    __name(this, "LovemilkTelemetry");
  }
  clients = [];
  constructor(ctx, config) {
    super(ctx, "lovemilkTelemetry", true);
  }
  async wavehand(client) {
    try {
      await client.disconnect();
      logger.debug("telemetry disconnect succceed");
    } catch (e) {
      logger.warn(`telemetry failed to disconnect:
${e.stack}`);
    }
  }
  async stop() {
    logger.debug("telemetry stop");
    while (this.clients.length > 0) {
      const client = this.clients.shift();
      await this.wavehand(client);
    }
  }
  createClient(pluginCtx, pluginConfig, enpoint) {
    const client = new Client(this.ctx, pluginCtx, pluginConfig, enpoint);
    this.clients.push(client);
    pluginCtx.on("dispose", async () => {
      await this.wavehand(client);
      this.clients = this.clients.filter((c) => c !== client);
    });
    return client;
  }
};
// Annotate the CommonJS export names for ESM import in node:
0 && (module.exports = {
  Client,
  ClientType,
  Config,
  LovemilkTelemetry,
  apply,
  getErrorDetail,
  inject,
  makeEvent,
  name
});

==> koishi-plugin-lovemilk-telemetry-0.0.2.tgz: package/lib/index.js <==
var __create = Object.create;
var __defProp = Object.defineProperty;
var __getOwnPropDesc = Object.getOwnPropertyDescriptor;
var __getOwnPropNames = Object.getOwnPropertyNames;
var __getProtoOf = Object.getPrototypeOf;
var __hasOwnProp = Object.prototype.hasOwnProperty;
var __name = (target, value) => __defProp(target, "name", { value, configurable: true });
var __export = (target, all) => {
  for (var name2 in all)
    __defProp(target, name2, { get: all[name2], enumerable: true });
};
var __copyProps = (to, from, except, desc) => {
  if (from && typeof from === "object" || typeof from === "function") {
    for (let key of __getOwnPropNames(from))
      if (!__hasOwnProp.call(to, key) && key !== except)
        __defProp(to, key, { get: () => from[key], enumerable: !(desc = __getOwnPropDesc(from, key)) || desc.enumerable });
  }
  return to;
};
var __toESM = (mod, isNodeMode, target) => (target = mod != null ? __create(__getProtoOf(mod)) : {}, __copyProps(
  // If the importer is in node compatibility mode or this is not an ESM
  // file that has been converted to a CommonJS file using a Babel-
  // compatible transform (i.e. "__esModule" has not been set), then set
  // "default" to the CommonJS "module.exports" for node compatibility.
  isNodeMode || !mod || !mod.__esModule ? __defProp(target, "default", { value: mod, enumerable: true }) : target,
  mod
));
var __toCommonJS = (mod) => __copyProps(__defProp({}, "__esModule", { value: true }), mod);

// src/index.ts
var src_exports = {};
__export(src_exports, {
  Client: () => Client,
  ClientType: () => ClientType,
  Config: () => Config,
  LovemilkTelemetry: () => LovemilkTelemetry,
  apply: () => apply,
  getErrorDetail: () => getErrorDetail,
  inject: () => inject,
  makeEvent: () => makeEvent,
  name: () => name,
  usage: () => usage
});
module.exports = __toCommonJS(src_exports);
var import_koishi3 = require("koishi");
var import_path = require("path");
var import_promises = __toESM(require("fs/promises"));

// src/telemetry/client.ts
var import_koishi2 = require("koishi");

// src/telemetry/types.ts
var ClientType = /* @__PURE__ */ ((ClientType2) => {
  ClientType2["koishi"] = "koishi";
  ClientType2["nonebot"] = "nonebot";
  return ClientType2;
})(ClientType || {});

// src/shared.ts
var import_koishi = require("koishi");
var import_node_machine_id = require("node-machine-id");
var logger = new import_koishi.Logger("lovemilk-telemetry");
var machineId = (0, import_node_machine_id.machineIdSync)(true);

// src/telemetry/client.ts
var defaultHeartbeatInterval = import_koishi2.Time.minute;
function makeEvent(name2, clientType, mid, data) {
  const now = /* @__PURE__ */ new Date();
  const UTCNow = new Date(
    now.getUTCFullYear(),
    now.getUTCMonth(),
    now.getUTCDate(),
    now.getUTCHours(),
    now.getUTCMinutes(),
    now.getUTCSeconds(),
    now.getUTCMilliseconds()
  );
  return {
    name: name2,
    localTime: now.getTime(),
    localUTCTime: UTCNow.getTime(),
    client: {
      type: clientType,
      machineID: mid,
      version: import_koishi2.version
      // imported from 'koishi'
    },
    data
  };
}
__name(makeEvent, "makeEvent");
function getErrorDetail(resp) {
  return resp.data ? resp.data["detail"] ?? resp.data["error"] ?? "" : "";
}
__name(getErrorDetail, "getErrorDetail");
var Client = class _Client {
  static {
    __name(this, "Client");
  }
  type = "koishi" /* koishi */;
  mid = machineId;
  pluginCtx;
  pluginConfig;
  ctx;
  static config;
  endpoint;
  connection = null;
  events = [];
  connected = false;
  disabled = false;
  static timeout = 3e4;
  heartbeatInterval = defaultHeartbeatInterval;
  heartbeatTimeoutID;
  isReconnecting = false;
  lastReconnect = 0;
  reconnectTried = 0;
  AutoReconnect = async (event = null, force = false) => {
    if (this.isReconnecting || this.disabled || !this.connection) {
      return;
    }
    this.isReconnecting = true;
    const core = /* @__PURE__ */ __name(async () => {
      logger.debug("telemetry connection lost, reconnecting...");
      try {
        await this.connect();
        this.isReconnecting = false;
      } catch (e) {
        this.lastReconnect = Date.now();
        this.reconnectTried += 1;
        this.isReconnecting = false;
        logger.debug(`telemetry reconnect failed
${e.stack}`);
        setTimeout(async () => {
          await this.AutoReconnect(event);
        }, 0);
      }
    }, "core");
    if (force) {
      this.ctx.setTimeout(async () => {
        await core();
      }, 0);
      return;
    }
    if (this.reconnectTried > _Client.config.telemetryMaxTry) {
      logger.warn(`telemetry reconnect failed, tried ${this.reconnectTried} times > ${_Client.config.telemetryMaxTry}, will retry in ${_Client.config.telemetryCoolDown} ms`);
      if (this.connection) {
        this.connection.removeEventListener("close", this.AutoReconnect);
      }
      this.connected = false;
      await (0, import_koishi2.sleep)(_Client.config.telemetryCoolDown);
      this.isReconnecting = false;
      this.reconnectTried = 0;
      this.lastReconnect = 0;
      await core();
      return;
    }
    const deltatime = Date.now() - this.lastReconnect;
    if (this.reconnectTried > _Client.config.telemetryUseTimeoutMoreThan) {
      if (deltatime < _Client.config.telemetryReconnectInterval) {
        logger.warn(`telemetry reconnect failed too many in ${this.reconnectTried} times, will retry in ${_Client.config.telemetryReconnectInterval - deltatime} ms`);
        this.connected = false;
        await (0, import_koishi2.sleep)(_Client.config.telemetryReconnectInterval - deltatime);
        await core();
        return;
      }
    } else {
      if (deltatime < _Client.timeout) {
        this.connected = false;
        await (0, import_koishi2.sleep)(_Client.timeout - deltatime);
      }
      await core();
      return;
    }
  };
  onSend = (_, __) => {
  };
  constructor(ctx, pluginCtx, pluginConfig, endpoint) {
    this.ctx = ctx;
    this.pluginCtx = pluginCtx;
    this.pluginConfig = pluginConfig;
    this.endpoint = new URL(endpoint);
  }
  static _setConfig(config) {
    _Client.config = config;
    _Client.timeout = _Client.config.axiosConfig ? _Client.config.axiosTimeout : 3e4;
  }
  isConnected() {
    return this.connected;
  }
  appendEventHistory(eventOrevents) {
    const events = Array.isArray(eventOrevents) ? eventOrevents : [eventOrevents];
    this.events.unshift(...events);
  }
  enable() {
    logger.success("telemetry is enabled");
    this.disabled = false;
  }
  disable() {
    logger.success("telemetry is disabled");
    this.disabled = true;
  }
  heartbeat() {
    if (!this.connected || this.disabled) {
      return;
    }
    this.heartbeatTimeoutID = setTimeout(async () => {
      if (this.connected) {
        try {
          await this.sendEvent("ping");
        } catch (e) {
          logger.error(`telemetry ping error:
${e.stack}`);
          if (!this.isReconnecting) {
            setTimeout(() => {
              this.AutoReconnect(null, true);
            }, 0);
          }
        }
      }
      if (this.connection) {
        this.heartbeat();
      }
    }, this.heartbeatInterval);
  }
  async connect() {
    if (this.disabled) {
      return;
    }
    if (this.connected) {
      this.connection.removeEventListener("close", this.AutoReconnect);
    }
    this.connection = await this.ctx.http.ws(this.endpoint, {
      timeout: _Client.timeout
    });
    return new Promise((resolve2, reject) => {
      const openListener = /* @__PURE__ */ __name(async (event) => {
        clearTimeout(timeoutID);
        this.reconnectTried = 0;
        this.lastReconnect = 0;
        this.connected = true;
        this.connection.removeEventListener("open", openListener);
        const closeListener = /* @__PURE__ */ __name(async (event2) => {
          if (this.connection) {
            this.connection.removeEventListener("close", closeListener);
          }
          if (event2.code === 1003) {
            logger.warn(`connection closed: ${event2.code}: ${event2.reason ?? "no reason"}`);
            await (0, import_koishi2.sleep)(_Client.config.telemetryCoolDown);
          }
          await this.AutoReconnect(event2);
        }, "closeListener");
        this.connection.addEventListener("close", closeListener);
        let resp;
        try {
          resp = await this.sendEvent("online", { isReconnect: this.isReconnecting, reconnectTimes: this.reconnectTried + (this.isReconnecting ? 1 : 0), config: this.pluginConfig });
        } catch (e) {
          reject(e);
          return;
        }
        if (resp.code !== 200) {
          reject(new Error(resp.msg, { cause: resp }));
          return;
        }
        this.heartbeatInterval = resp.data?.heartbeat_interval * 1e3 / 2;
        logger.debug(`heartbeat interval set: ${this.heartbeatInterval} ms`);
        logger.debug("telemetry handshake succeed");
        this.heartbeat();
        resolve2(resp);
      }, "openListener");
      this.connection.addEventListener("open", openListener);
      const errorListener = /* @__PURE__ */ __name(async (event) => {
        clearTimeout(timeoutID);
        this.connected = false;
        this.connection.removeEventListener("error", errorListener);
        await this.AutoReconnect(event);
        reject(event.error);
      }, "errorListener");
      this.connection.addEventListener("error", errorListener);
      const timeoutID = setTimeout(() => {
        this.connection.removeEventListener("open", openListener);
        reject(new Error(`waiting server response timeout of ${_Client.timeout} ms`));
      }, _Client.timeout);
    });
  }
  async disconnect(code, reason) {
    if (this.disabled) {
      return;
    }
    this.disabled = true;
    if (this.connection) {
      this.connection.removeEventListener("close", this.AutoReconnect);
      this.connection.removeEventListener("error", this.AutoReconnect);
      clearTimeout(this.heartbeatTimeoutID);
      let wavehandError = void 0;
      try {
        await this.sendEvent("offline", { config: this.pluginConfig }, true);
      } catch (e) {
        wavehandError = e;
        logger.warn(`telemetry failed to wavehand:
${e.stack}`);
      }
      this.connected = false;
      await this.connection.close(code, reason);
      return new Promise((resolve2, reject) => {
        wavehandError === void 0 ? resolve2() : reject(wavehandError);
      });
    }
  }
  async sendEvent(name2, data, ignoreResp = false) {
    if (this.disabled) {
      return;
    }
    const event = makeEvent(name2, this.type, this.mid, data);
    this.events.push(event);
    this.onSend(event, this.events);
    if (!this.connected) {
      logger.warn(`telemetry client is not connected while sending ${name2} event`);
      return;
    }
    logger.debug("telemetry ↑", event);
    await this.connection.send(JSON.stringify(event));
    if (ignoreResp) {
      return;
    }
    return new Promise((resolve2, reject) => {
      const messageListener = /* @__PURE__ */ __name((event2) => {
        clearTimeout(timeoutID);
        if (this.connection) {
          this.connection.removeEventListener("message", messageListener);
        }
        try {
          const received = JSON.parse(event2.data);
          logger.debug("telemetry ↓", received);
          resolve2(received);
        } catch (e) {
          reject(e);
        }
      }, "messageListener");
      this.connection.addEventListener("message", messageListener);
      const errorListener = /* @__PURE__ */ __name((event2) => {
        clearTimeout(timeoutID);
        if (this.connection) {
          this.connection.removeEventListener("error", errorListener);
        }
        reject(event2.error);
      }, "errorListener");
      this.connection.addEventListener("error", errorListener);
      const timeoutID = setTimeout(() => {
        if (this.connection) {
          this.connection.removeEventListener("message", messageListener);
          this.connection.removeEventListener("error", errorListener);
        }
        reject(new Error(`waiting server response timeout of ${_Client.timeout} ms`));
      }, _Client.timeout);
    });
  }
};

// src/index.ts
var name = "lovemilk-telemetry";
var inject = ["http", "console"];
var usage = `<a class="el-button" href="/lovemilk-telemetry/EULA">Lovemilk Telemetry 终端用户协议 & 隐私政策</a>`;
var isDev = process.env.NODE_ENV === "development";
var Config = import_koishi3.Schema.intersect([
  import_koishi3.Schema.object({
    telemetryReconnectInterval: import_koishi3.Schema.number().min(1).max(import_koishi3.Time.day).default(import_koishi3.Time.minute * 15).description("遥测重连间隔 (毫秒)").step(1).hidden(!isDev),
    telemetryUseTimeoutMoreThan: import_koishi3.Schema.number().min(3).default(10).description("当遥测连续重连失败 (不包括首次连接) 大于该值后, 使用 `telemetryReconnectInterval` 毫秒 遥测重连间隔, 否则, 使用超时时间作为重连间隔").step(1).hidden(!isDev),
    telemetryMaxTry: import_koishi3.Schema.number().min(10).default(20).description("当遥测连续重连失败 (不包括首次连接) 大于该值后, 暂停 `telemetryCoolDown` 毫秒后重制重连状态").step(1).hidden(!isDev),
    telemetryCoolDown: import_koishi3.Schema.number().min(import_koishi3.Time.minute * 5).default(import_koishi3.Time.hour * 12).description("当遥测连续重连失败 (不包括首次连接) 大于 `telemetryMaxTry` 后, 暂停 该值(毫秒) 后重制重连状态").step(1).hidden(!isDev)
  }).description("遥测配置"),
  import_koishi3.Schema.object({
    axiosConfig: import_koishi3.Schema.boolean().default(false).description("启用自定义请求配置 <br>> 不启用将使用默认请求配置")
  }).description("请求配置"),
  import_koishi3.Schema.union([
    import_koishi3.Schema.object({
      axiosConfig: import_koishi3.Schema.const(true).required(),
      axiosTimeout: import_koishi3.Schema.number().default(3e5).description("请求超时 毫秒")
    }),
    import_koishi3.Schema.object({})
  ])
]);
async function apply(ctx, config) {
  logger.debug(`machine ID: ${machineId}`);
  const EULAContent = await import_promises.default.readFile((0, import_path.resolve)(__dirname, "../markdown/EULA.md"), "utf-8");
  Client._setConfig(config);
  ctx.inject(["console"], (ctx2) => {
    ctx2.console.addEntry({
      dev: (0, import_path.resolve)(__dirname, "../client/index.ts"),
      prod: (0, import_path.resolve)(__dirname, "../dist")
    });
  });
  ctx.console.addListener("lovemilk-telemetry/eula", () => {
    return EULAContent;
  });
  ctx.plugin(LovemilkTelemetry, config);
}
__name(apply, "apply");
var LovemilkTelemetry = class extends import_koishi3.Service {
  static {
    __name(this, "LovemilkTelemetry");
  }
  clients = [];
  constructor(ctx, config) {
    super(ctx, "lovemilkTelemetry", true);
  }
  async wavehand(client) {
    try {
      await client.disconnect();
      logger.debug("telemetry disconnect succceed");
    } catch (e) {
      logger.warn(`telemetry failed to disconnect:
${e.stack}`);
    }
  }
  async stop() {
    logger.debug("telemetry stop");
    while (this.clients.length > 0) {
      const client = this.clients.shift();
      await this.wavehand(client);
    }
  }
  createClient(pluginCtx, pluginConfig, enpoint) {
    const client = new Client(this.ctx, pluginCtx, pluginConfig, enpoint);
    this.clients.push(client);
    pluginCtx.on("dispose", async () => {
      await this.wavehand(client);
      this.clients = this.clients.filter((c) => c !== client);
    });
    return client;
  }
};
// Annotate the CommonJS export names for ESM import in node:
0 && (module.exports = {
  Client,
  ClientType,
  Config,
  LovemilkTelemetry,
  apply,
  getErrorDetail,
  inject,
  makeEvent,
  name,
  usage
});
